#!/usr/bin/env python

import os
import sys

# Directory containing the FASTQ files
directory = snakemake.input.fastp
genome_size = snakemake.params.gensize

# Get all files in the directory
files = os.listdir(directory)

# Filter for only FASTQ files
fastq_files = [file for file in files if file.endswith(".fastq") or file.endswith(".fastq.gz")]

# Process each FASTQ file to generate data
data = []
for fastq_file in fastq_files:
    # Extract the sample name (text before ".fastq")
    sample_name = fastq_file.split(".fastq")[0]
    sample_name += ".fastq"
    filepath = os.path.join(directory, fastq_file)
    # Append data to the list
    data.append((sample_name, filepath, genome_size))

# Write the data to a file
with open(snakemake.output.csv, "w") as file:

    # Write each line of data
    for entry in data:
        file.write(f"{entry[0]},{entry[1]},{entry[2]}\n")



import argparse
import pandas as pd

def parse_files(file_list):
    """Parse information from the given list of files and return a structured list."""
    data_list = []

    # Loop through each provided file
    for file_path in file_list:
        with open(file_path, "r") as f:
            lines = f.readlines()

        # Extract sample name from filename (excluding path)
        sample_name = file_path.split("/")[-1]

        # Initialize dictionary for the current sample
        sample_data = {"Sample": sample_name}

        # Extract key-value pairs
        for line in lines:
            line = line.strip()
            if line and ":" in line:
                key, value = line.split(":", 1)
                key = key.strip().replace(" ", "_").lower()  # Format column names
                value = value.strip()
                sample_data[key] = value

        # Append parsed data
        data_list.append(sample_data)

    return data_list

def save_to_tsv(data_list, output_file):
    """Save parsed data to a TSV file."""
    df = pd.DataFrame(data_list)
    df.to_csv(output_file, sep="\t", index=False)
    print(f"TSV file saved as {output_file}")

def main():
    parser = argparse.ArgumentParser(description="Parse genome annotation files and save to TSV.")
    parser.add_argument("-i", "--input_files", nargs='+', required=True, help="List of input files to process.")
    parser.add_argument("-o", "--output_file", required=True, help="Path to save the output TSV file.")

    args = parser.parse_args()

    # Process files
    data_list = parse_files(args.input_files)

    # Save results
    save_to_tsv(data_list, args.output_file)

if __name__ == "__main__":
    main()



"""
Entrypoint for sphaehost

Check out the wiki for a detailed look at customising this file:
https://github.com/beardymcjohnface/Snaketool/wiki/Customising-your-Snaketool
"""

import os
import click

from snaketool_utils.cli_utils import OrderedCommands, run_snakemake, copy_config, echo_click


def snake_base(rel_path):
    """Get the filepath to a Snaketool system file (relative to __main__.py)"""
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), rel_path)


def get_version():
    """Read and print the version from the version file"""
    with open(snake_base("baczy.VERSION"), "r") as f:
        version = f.readline()
    return version


def print_citation():
    """Read and print the Citation information from the citation file"""
    with open(snake_base("baczy.CITATION"), "r") as f:
        for line in f:
            echo_click(line)


def default_to_output(ctx, param, value):
    """Callback for click options; places value in output directory unless specified"""
    if param.default == value:
        return os.path.join(ctx.params["output"], value)
    return value


def common_options(func):
    """Common command line args
    Define common command line args here, and include them with the @common_options decorator below.
    """
    options = [
        click.option("--output",help="Output directory",
            type=click.Path(),
            default="baczy.out",
            show_default=True,
        ),
        click.option(
            "--configfile",
            default="config.yaml",
            show_default=False,
            callback=default_to_output,
            help="Custom config file [default: (outputDir)/config.yaml]",
        ),
        click.option(
            "--threads", help="Number of threads to use", default=1, show_default=True
        ),
        click.option(
            "--profile",
            default=None,
            help="Snakemake profile to use",
            show_default=False,
        ),
        click.option(
            "--use-conda/--no-use-conda",
            default=False,
            help="Use conda for Snakemake rules",
            show_default=True,
        ),
        click.option(
            "--use-singularity/--no-use-singularity",
            default=True,
            help="Use containers for Snakemake rules",
            show_default=True,
        ),
        click.option(
            "--conda-prefix",
            default=snake_base(os.path.join("workflow", "conda")),
            help="Custom conda env directory",
            type=click.Path(),
            show_default=False,
        ),
        click.option(
            "--snake-default",
            multiple=True,
            default=[
                "--printshellcmds",
                "--nolock",
                "--show-failed-logs",
            ],
            help="Customise Snakemake runtime args",
            show_default=True,
        ),
        click.option(
            "--log",
            default="baczy.log",
            callback=default_to_output,
            hidden=True,
        ),
        click.option(
            "--system-config",
            default=snake_base(os.path.join("config", "config.yaml")),
            hidden=True,
        ),
        click.argument("snake_args", nargs=-1),
    ]
    for option in reversed(options):
        func = option(func)
    return func


@click.group(
    cls=OrderedCommands, context_settings=dict(help_option_names=["-h", "--help"])
)
@click.version_option(get_version(), "-v", "--version", is_flag=True)
def cli():
    """Bacterial asembly and annotation
    \b
    For more options, run:
    baczy command --help"""
    pass


help_msg_extra = """
\b
CLUSTER EXECUTION:
baczy run ... --profile [profile]
For information on Snakemake profiles see:
https://snakemake.readthedocs.io/en/stable/executing/cli.html#profiles
\b
"""

@click.command(epilog=help_msg_extra, context_settings=dict(help_option_names=["-h", "--help"], ignore_unknown_options=True))
@common_options
def install(**kwargs):
    """The install function for databases"""
    merge_config = {
        'baczy': {
            'args': kwargs   
        }
    }

    # run!
    run_snakemake(
        snakefile_path=snake_base(os.path.join('workflow', 'install.smk')),
        merge_config=merge_config,
        **kwargs
    )

@click.command(
    epilog=help_msg_extra,
    context_settings=dict(
        help_option_names=["-h", "--help"], ignore_unknown_options=True
    ),
)
@click.option("--input", "_input", help="Input file/directory", type=str, required=True)
@click.option('--sequencing', "sequencing", help="sequencing method", default='paired', show_default=True,
                type=click.Choice(['paired', 'longread']))
@common_options
def run(**kwargs):
    """Run baczy"""
    # Config to add or update in configfile
    merge_config = {
        "baczy": {
            "args": kwargs
        }
    }

    # run!
    run_snakemake(
        # Full path to Snakefile
        snakefile_path=snake_base(os.path.join("workflow", "Snakefile")),
        merge_config=merge_config,
        **kwargs
    )


@click.command()
@click.option('--configfile', default='config.yaml', help='Copy template config to file', show_default=True)
@click.option(
    '--system-config',
    default=lambda: snake_base(os.path.join("config", "config.yaml")),
    hidden=True,
    help="System config file"
)
def config(configfile, system_config, **kwargs):
    """Copy the system default config file"""
    copy_config(configfile, system_config=system_config)


@click.command()
def citation(**kwargs):
    """Print the citation(s) for this tool"""
    print_citation()


cli.add_command(run)
cli.add_command(config)
cli.add_command(citation)

def main():
    cli()


if __name__ == "__main__":
    main()
